//
// Created by zhengjiazhu on 12/25/19.
//

#ifndef IOT_DIAGNOSE_H
#define IOT_DIAGNOSE_H

#ifdef __cplusplus
extern "C" {
#endif

#include <cJSON.h>

/**
 * ICMP协议相关探测指标
 */
typedef struct IotDiagnoseLink {
    float delay; /** @brief 网络延迟 */
    float shake; /** @brief 网络抖动 */
    int loss; /** @brief 丢包率 */
    int ttl; /** @brief 路由节点数量 */
    float max; /** @brief 最大延迟 */
    float min; /** @brief 最小延迟 */
    float router; /** @brief 路由延迟 */
} IotDiagnoseLink_t;

/**
 * HTTP网络协议相关探测指标
 */
typedef struct IotDiagnosePro {
    float lookup; /** @brief DNS解析耗时 */
    float connect; /** @brief HTTP连接耗时 */
    float start; /** @brief HTTP首包耗时 */
    float ssl; /** @brief SSL/SSH握手耗时 */
    char *addr; /** @brief DNS服务器地址 */
    char *result; /** @brief DNS解析结果 */
} IotDiagnosePro_t;

typedef struct IotDiagnoseCallback {

    /**
     * Ping诊断实时回调诊断结果回调
     * @param delay 延迟
     * @param avg 平均延迟
     * @param loss 丢包率
     * @param ttl 路由节点数量
     */
    void (*diagnosePingProgress)(float delay, float avg, int loss, int ttl);

    /**
     * Ping诊断反馈诊断完成的结果回调
     * @param min 最小延迟
     * @param max 最大延迟
     * @param avg 平均延迟
     * @param loss 丢包率
     */
    void (*diagnosePingResult)(float min, float max, float avg, int loss);

    /**
     * 诊断结论回调
     */
    void (*diagnoseResult)(IotDiagnoseLink_t link, IotDiagnosePro_t pro);
} IotDiagnoseCallback_t;

int IotDiagnose_Notice(cJSON *data, char **msg);

int IotDiagnose_MonitorConfig(int mode, int period);

/**
 * 本地发起Ping诊断
 * @param host 域名
 * @param period 诊断周期，秒
 * @return
 */
int IotDiagnose_Ping(char *host, int period, IotDiagnoseCallback_t callback);

/**
 * 本地发起网络诊断
 * @param host 域名
 * @param port 端口
 * @param type 0:http诊断 1:https诊断
 * @param range 0:全部诊断 1:tcp诊断 2:http诊断
 * @param period 诊断周期，秒
 * @return
 */
int IotDiagnose_Process(char *host, int port, int type, int range, int period, IotDiagnoseCallback_t callback);

#ifdef __cplusplus
}
#endif

#endif //IOT_DIAGNOSE_H
