//
// Created by root on 8/5/19.
//

#ifndef IOT_REMOTE_CONFIG_H
#define IOT_REMOTE_CONFIG_H

#include "cJSON.h"

typedef enum IotRemoteConfigStatus {
    IOT_REMOTE_CONFIG_STATUS_SUCCESS = 2, /** @ref 远程配置成功 */
    IOT_REMOTE_CONFIG_STATUS_FAILURE = 3, /** @ref 远程配置失败 */
} IotRemoteConfigStatus_t;

typedef struct IotRemoteConfigCallback {

    /**
     * @brief 远程配置版本上报回调
     * @param result true: 成功 false:失败
     * @param msg 描述信息
     */
    void (*cbRemoteVersionResponse)(bool result, char *msg);

    /**
     * @brief 远程配置上报状态回调
     * @param result true: 成功 false:失败
     * @param msg 描述信息
     */
    void (*cbRemoteStatusResponse)(bool result, char *msg);

    /** @ref 远程配置回调 */
    int (*cbRemoteNotice)(cJSON *config, int version, char **msg);
} IotRemoteConfigCallback_t;

typedef struct IotRemoteConfig {
    bool isOpenRemoteConfig;
    IotRemoteConfigCallback_t cb;
} IotRemoteConfig_t;

IotRemoteConfig_t iotRemoteConfig;

int IotRemoteConfig_Init(bool isOpen, IotRemoteConfigCallback_t cb);

int IotRemoteConfig_UploadVersion(int version);

/** @ref  远程配置状态上报
 *
 * @param appStatus
 * @param sysStatus
 */
int IotRemoteConfig_UploadStatus(IotRemoteConfigStatus_t appStatus, int version, char *msg);

/** @brief 上报远程配置版本响应
 *
 * @param data 响应参数
 */
int IotRemoteConfig_UploadVersionResponse(cJSON *data);

/** @brief 上报远程配置状态响应
 *
 * @param data 响应参数
 */
int IotRemoteConfig_UploadStatusResponse(cJSON *data);

/** @ref  远程配置通知
 *
 * @param inParams
 * @param msg
 */
int IotRemoteConfig_Notice(cJSON *inParams, char **msg);

#endif //IOT_REMOTE_CONFIG_H
